using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace CameraAppCE
{
    /**
     * This class takes care of interaction between the application 
     * (managed code) and the various WinCE API it needs 
     * (camera driver, sound API, etc...)
     */ 
    class CEApiInvoke
    {
        [DllImport("Coredll.dll")]
        public static extern IntPtr CreateFile
		(
			String lpFileName,
			UInt32 dwDesiredAccess,
			UInt32 dwShareMode,
            UInt32 lpSecurityAttributes,
            UInt32 dwCreationDisposition,
			UInt32 dwFlagsAndAttributes,
            UInt32 hTemplateFile
		);
        [DllImport("Coredll.dll")]
        public static extern int CloseHandle(IntPtr handle);
        [DllImport("Coredll.dll")]
        public static extern int DeviceIoControl
        (
            IntPtr handle,
            int dwIoControlCode,
            UInt32 lpInBuffer,
            UInt32 nInBufferSize,
            UInt32 lpOutBuffer,
            UInt32 nOutBufferSize,
            UInt32 lpBytesReturned,
            UInt32 lpOverlapped 
        );

        public const int SND_SYNC = 0x0;    // play synchronously (default)
        public const int SND_ASYNC = 0x1;   // play asynchronously
        [DllImport("coredll.dll", EntryPoint = "PlaySoundW", SetLastError = true)]
        public extern static int PlaySoundW(String lpszName, int hModule, int
        dwFlags);

        [DllImport("coredll.dll")]
        private static extern IntPtr FindWindow(string lpClassName, string
        lpWindowName);
        [DllImport("coredll.dll")]
        private static extern IntPtr ShowWindow(IntPtr hWnd, int visible);

        public static void enableStartButton(bool enable)
        {
            int visible = (enable) ? 1 : 0;            
            ShowWindow(FindWindow("HHTaskBar", null), visible); 
        }

        private const int FILE_DEVICE_UNKNOWN = 0x22;
        private const int CAMERA_IOCTL_INDEX = 0x0840;        
        private static int calculateIOCTLCode(int deviceIndex, int ioctlIndex)
        {
            return (FILE_DEVICE_UNKNOWN << 16 | (deviceIndex + ioctlIndex) << 2);
        }
        private static int calculateCameraIOCTLCode(int ioctlIndex)
        {
            return calculateIOCTLCode(CAMERA_IOCTL_INDEX, ioctlIndex);
        }

        public const int CAMERA_IOCTL_START_PREVIEW_INDEX = 0x1;
        public const int CAMERA_IOCTL_STOP_PREVIEW_INDEX = 0x2;  //CAM_CUSTOM_IOCTL_STOP in cam_driver_ioctl.h
        public const int CAMERA_IOCTL_TAKE_SNAPSHOT_INDEX = 0x3;
         

        static IntPtr hCameraDriver; 
        static bool isCameraDriverHandleOpen = false;
        private const String strCameraDeviceName = "CAM1:";
        public static void openCameraHandle()
        {
            if (isCameraDriverHandleOpen)
                return;
            hCameraDriver = CreateFile(strCameraDeviceName, 0, 0, 0, 0, 0, 0);
            isCameraDriverHandleOpen = true;
        }
        public static void closeCameraHandle()
        {
            if (!isCameraDriverHandleOpen)
                return;
            CloseHandle(hCameraDriver);
            isCameraDriverHandleOpen = false;
        }
        public static void sendCameraIOCTL(int ioctl)
        {
            openCameraHandle();
            DeviceIoControl(hCameraDriver, calculateCameraIOCTLCode(ioctl), 0, 0, 0, 0, 0, 0);
        }

    }
}
