using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace CameraAppCE
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            //Since we open an overlay, we don't allow the user to use the
            //start button and taskbar while the application is running
            CEApiInvoke.enableStartButton(false);            
            
            /**
             * This sequence is required to work around a bug in the driver
             * The picture starts frozen without the extra stop-start...
             * should be fixed...
            */
            startButton_Click(null, null);
            stopButton_Click(null, null);
            startButton_Click(null, null);
        }
        private static bool isPreviewWindowActive = false;
        
        private const String SNAPSHOT_WAVE_SOUND_PATH = "\\Windows\\snapshot.wav";
        private void playSnapshotSound()
        {
            CEApiInvoke.PlaySoundW(SNAPSHOT_WAVE_SOUND_PATH, 0, CEApiInvoke.SND_ASYNC);
        }
        
        // Starts the preview window (in previous implementation, 
        //there was a "Start Preview" button)
        private void startButton_Click(object sender, EventArgs e)
        {
            if (isPreviewWindowActive)
                return;
            isPreviewWindowActive = true; 
            CEApiInvoke.sendCameraIOCTL(CEApiInvoke.CAMERA_IOCTL_START_PREVIEW_INDEX);
            
        }
        /**
         * Stops the preview window
         */
        private void stopButton_Click(object sender, EventArgs e)
        {
            if (!isPreviewWindowActive)
                return;
            isPreviewWindowActive = false;
            CEApiInvoke.sendCameraIOCTL(CEApiInvoke.CAMERA_IOCTL_STOP_PREVIEW_INDEX);        
        }
        private const String defaultImageLocation = "\\My Documents";
        private Thread snapshotThread = null;

        private void snapshotThreadProc()
        {   
            playSnapshotSound();
            CEApiInvoke.sendCameraIOCTL(CEApiInvoke.CAMERA_IOCTL_TAKE_SNAPSHOT_INDEX);
            
            ImageManipulator manip = new ImageManipulator(defaultImageLocation);
            manip.convertFileFormat();
            isSnapshotInProgress = false;

            /**
             * this will enable the snapshot button and write 
             * "Picture saved in \\My Documents" in the message box
             * it's not allowed for a non-UI thread to do it directly
             */
            object[] pList = { this, System.EventArgs.Empty };
            messageBox.BeginInvoke(new System.EventHandler(UpdateUIFinished), pList);                    
        }

        private void UpdateUIFinished(object o, System.EventArgs e)
        {
            // Now OK - this method will be called via
            // Control.Invoke, so we are allowed to do
            // things to the UI.
            messageBox.Text = "Picture saved in \\My Documents";
            buttonSanpshot.Enabled = true;
        }


        private bool isSnapshotInProgress = false;

        private void buttonSanpshot_Click(object sender, EventArgs e)
        {
            if (isSnapshotInProgress)
                return;
            isSnapshotInProgress = true;
            buttonSanpshot.Enabled = false;
            messageBox.Text = "Processing...";                        
            //The following thread will do most of the work...
            snapshotThread = new Thread(new ThreadStart(snapshotThreadProc));
            snapshotThread.Start();
        }

        private void buttonExit_Click(object sender, EventArgs e)
        {
            stopButton_Click(sender, e);
            CEApiInvoke.enableStartButton(true);
            Application.Exit();
        }


    }
}